# You can use it as you like, but I give no waranty that it works.
# Use it on your own risk.

#!/bin/bash

function usage
{
  echo " -c | --config <configuration> : default p0b32p1b32"
  echo " -i | --interactive  default:batch"
  echo " -h | --help"
  echo " -d | --designentry [verilog|vhdl]  default:verilog"
  echo " -p | --donotpatch  default:patch"
  echo " -t | --template   default:none"
  echo

  echo
  echo " -c p032p132 to generate config p032p132"
}

function showhelp
{
    usage
    echo
    echo "some more advanced description here"
    echo
    echo "generate a new configuration based on an existing one:"
    echo "./runimg -i -c myconfig -t p0b32p1b32"
}

if [ -z "$XILINX" ]
then
  echo
  echo " XILINX environment variable is not set."
  echo " please run your envirnment setup script provided by the XILINX toolchain"
  echo " by sourcing the settings32.sh script or the settings32.csh script:"
  echo
  echo "         . /opt/Xilinx/13.2/ISE_DS/settings32.sh"
  echo
  echo " where /opt/Xilinx/13.2 is an example of the XILINX ISE install path"
  echo

  exit 1
fi

CONFIG=p0b32p1b32p2r32
GENMODE="batch"
designentry="verilog"
donotpatch=0
template=""

while [ "$1" != "" ]; do
    case $1 in
        -c | --config )         shift
                                CONFIG="$1"
                                ;;
        -i | --interactive )    GENMODE="interactive"
                                ;;
        -d | --designentry )
                                shift
                                designentry="$1"
                                if [ "$designentry" != "verilog" -a "$designentry" != "vhdl" ]
                                then
                                    usage
                                    exit
                                fi
                                ;;
        -p | --donotpatch )
                                donotpatch=1
                                ;;
        -t | --template )       shift
                                template="$1"
                                ;;
        -h )                    usage
                                exit
                                ;;
        --help )                showhelp
                                exit
                                ;;
        * )                     usage
                                exit 1
    esac
    shift
done

echo "config :" $CONFIG
echo "genmode:" $GENMODE
echo "designentry:" $designentry


export XILINXMIG="$XILINX"/coregen/ip/xilinx/other/com/xilinx/ip/mig_v3_8/bin/lin/mig

export GENDIR="gen_"$designentry"_"$CONFIG
export CONFIGFILE="$GENDIR"/config.txt

mkdir -p "$GENDIR"

vhdlsim="false"
verilogsim="true"
if [ "$designentry" = "vhdl" ]
then
  vhdlsim="true"
  verilogsim="false"
fi

PROJECTFILE="mig_xc6slx25-ftg256-3-""$CONFIG"".prj"
NEWPROJECTFILE=""

if [ -n "$template" ]
then
  if [ -f "$PROJECTFILE" ]
  then
    echo
    echo "project file to generate allready exists."
    echo "please remove it manually, if that is what you want"
    echo
  fi

  NEWPROJECTFILE="$PROJECTFILE"
  PROJECTFILE="mig_xc6slx25-ftg256-3-""$template"".prj"
fi


if [ ! -f "$PROJECTFILE" ]
then
  echo
  echo "project file $PROJECTFILE is missing"
  echo "to generate it use parameter -i -t p0b32p1b32 -c $CONFIG to generate it interactively"
  echo
  exit
fi


echo "SET_PREFERENCE subworkingdirectory "$GENDIR > "$CONFIGFILE"
echo "SET_PREFERENCE outputdirectory ddr2_mig" >> "$CONFIGFILE"
echo "SET_FLAG debug FALSE" >> "$CONFIGFILE"
# SET_FLAG mode [batch|interactive]
echo "SET_FLAG mode "$GENMODE >> "$CONFIGFILE"
echo "SET_FLAG STANDALONE_MODE TRUE" >> "$CONFIGFILE"
echo "SET_PREFERENCE devicefamily spartan6" >> "$CONFIGFILE"
echo "SET_PREFERENCE device xc6slx25" >> "$CONFIGFILE"
echo "SET_PREFERENCE package ftg256" >> "$CONFIGFILE"
echo "SET_PREFERENCE speedgrade -3" >> "$CONFIGFILE"
echo "SET_PREFERENCE vhdlsim $vhdlsim" >> "$CONFIGFILE"
echo "SET_PREFERENCE verilogsim $verilogsim" >> "$CONFIGFILE"
echo "SET_PREFERENCE badmodulenames top addr_gen main top2 top4" >> "$CONFIGFILE"
echo "SET_PREFERENCE designentry $designentry" >> "$CONFIGFILE"
echo "SET_PREFERENCE flowvendor foundation_ise" >> "$CONFIGFILE"
echo "SET_PARAMETER component_name ddr2_mig" >> "$CONFIGFILE"
echo "SET_PARAMETER xml_input_file "$PROJECTFILE >> "$CONFIGFILE"

if "$XILINXMIG" -cg_exc_inp "$CONFIGFILE" -cg_exc_out "$GENDIR"/ddr2_mig_output.txt
then
  echo "generated memory core "$CONFIG
else
  echo "Failed to generate memory core "$CONFIG"."
fi

function applypatch
{
    filetocheck="$1"
    patchfile="$2"

    rtl_dir="$GENDIR""/ddr2_mig/user_design/rtl/"
    rtl_backpath="../../../../"

    [ -f "$rtl_dir""$filetocheck" ] && ( cd "$rtl_dir" && patch -p0 < "$rtl_backpath""$patchfile" )
}

ORIGINAL_PROJECT="$GENDIR""/ddr2_mig/user_design/mig.prj"
if [ -n "$NEWPROJECTFILE" -a -f "$ORIGINAL_PROJECT" -a ! -f "$NEWPROJECTFILE" ]
then
  cp "$ORIGINAL_PROJECT" "$NEWPROJECTFILE"
fi

if [ "$donotpatch" -eq 0 ]
then
    if [ -f "patchlist_""$designentry"".txt" ]
    then
        for i in $(cat "patchlist_""$designentry"".txt" )
        do
            applypatch $(echo $i | cut -d':' -f1) $(echo $i | cut -d':' -f2)
        done
    fi
fi

#
# create a xst-prj file
# cat gen_verilog_p0b32p1b32p2r32/ddr2_mig/user_design/synth/ddr2_mig.prj | sed -e 's/\.\./generated\/ddr2_mig\/gen_verilog_p0b32p1b32p2r32\/ddr2_mig\/user_design/g'
#